#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import os
import urllib2
import urllib
import json
import time
import datetime
import codecs
import collections
import traceback

import logger
log = logger.getLogger("checkLogs")

infoFile = "/var/lib/cashserver/zabbix-agent/checkLogs.json"
messagesFileSkeleton = "/var/lib/cashserver/zabbix-agent/%s_messages.log"
messagesFilePrefix = ""

levels = [ "ERROR", "WARN" ]
types = [ "GENERATION", "EXCHANGE_SALES", "EXCHANGE_DICTIONARIES" ]

log.info("begin logs check...")

restLogCountPathSkeleton = "http://localhost:8080/CSrest/rest/cashes/log/count?level=%s&sortparam=SORT_BY_TIME&type=%s"
restLogPathSkeleon = "http://localhost:8080/CSrest/rest/cashes/log?level=%s&sortparam=SORT_BY_TIME&type=%s&page=0&size=%d"

params = None

if os.path.exists(infoFile):
    with open(infoFile) as infile:
        params = json.load(infile)
else:
    params = {}
    for level in levels:
        levParam = {}
        params[level] = levParam
        for logType in types:
            levParam[logType] = {'date': time.time(), 'count': 0}

log.info("input params: %s", params)

def getDaysDelta(d1,d2):
    etalon = datetime.datetime(2010, 1, 1, 00, 00, 00)
    return (d2 - etalon).days - (d1 - etalon).days

def execGet(uri):
    result = None
    log.debug("uri: %s", uri)
    try:
        result = urllib2.urlopen(uri).read()
    except Exception, e:
        log.warn(u"exception while get data: %s", unicode(e))
    return result

try:
    for level in levels:
        for logType in types:
            log.debug("level: %s, type: %s", level, logType)
            uriCount = execGet(restLogCountPathSkeleton % (level, logType))
            if uriCount:
                count = int(uriCount)
                currentTime = datetime.datetime.now()
                saveTime = datetime.datetime.fromtimestamp(int(params[level][logType]['date']))
                itsToday = getDaysDelta(currentTime, saveTime) == 0
                newMessagesCount = 0
                messagesStrings = []
                if itsToday:
                    newMessagesCount = max(0, count - params[level][logType]['count'])
                else:
                    log.debug("get yesterday logs...")
                    log.info("remove yesterday file...")
                    f = codecs.open(messagesFileSkeleton % (messagesFilePrefix + level + logType), 'w', 'UTF-8')
                    f.close()
                    log.info("yesterday file removed.")
                    uriForYestCount = restLogCountPathSkeleton % (level, logType) + "&date=" + saveTime.strftime('%d.%m.%Y') + "+12%%3A00%%3A00"
                    log.debug("uri for yesterday count: %s", uriForYestCount)
                    yesterdayCount = execGet(uriForYestCount)
                    yestMessagesCount = max(0, int(yesterdayCount) - params[level][logType]['count'])
                    uriForYestLogs = restLogPathSkeleon % (level, logType, yestMessagesCount) + "&date=" + saveTime.strftime('%d.%m.%Y') + "+12%%3A00%%3A00"
                    log.debug("uri for yesterday logs: %s", uriForYestLogs)
                    messagesStrings.append(execGet(uriForYestLogs))
                    newMessagesCount = count
                
                messagesStrings.append(execGet(restLogPathSkeleon % (level, logType, newMessagesCount)))
                if messagesStrings:
                    f = codecs.open(messagesFileSkeleton % (messagesFilePrefix + level + logType), 'ab', 'UTF-8')
                    messages = []
                    for m in messagesStrings:
                        messages += json.loads(m)
                    for message in sorted(messages, key = lambda m: m['date']):
                        mDate = message['date']
                        mCode = message['code']
                        mBody = message['formattedMessage']
                        log.debug("incoming message: %s" , message)
                        f.write( u'%s %s: %s'  % (mDate, mCode, mBody) + '\n')
                    f.close()
                    params[level][logType] = {'date': time.mktime(currentTime.timetuple()), 'count': count}
except Exception, e:
    log.error(u"exception while get data: %s", unicode(e))
    traceback.print_exc()

log.info("output params: %s", params)
with open(infoFile, 'w') as outfile:
    json.dump(params, outfile)

log.info("end logs check")